@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2026 The OpenSSL Project Authors. All Rights Reserved.
+ * Copyright 2019-2025 The OpenSSL Project Authors. All Rights Reserved.
  *
  * Licensed under the Apache License 2.0 (the "License").  You may not use
  * this file except in compliance with the License.  You can obtain a copy
@@ -46,12 +46,12 @@ static OSSL_FUNC_asym_cipher_set_ctx_params_fn rsa_set_ctx_params;
 static OSSL_FUNC_asym_cipher_settable_ctx_params_fn rsa_settable_ctx_params;
 
 static OSSL_ITEM padding_item[] = {
-    { RSA_PKCS1_PADDING, OSSL_PKEY_RSA_PAD_MODE_PKCSV15 },
-    { RSA_NO_PADDING, OSSL_PKEY_RSA_PAD_MODE_NONE },
-    { RSA_PKCS1_OAEP_PADDING, OSSL_PKEY_RSA_PAD_MODE_OAEP }, /* Correct spelling first */
-    { RSA_PKCS1_OAEP_PADDING, "oeap" },
-    { RSA_X931_PADDING, OSSL_PKEY_RSA_PAD_MODE_X931 },
-    { 0, NULL }
+    { RSA_PKCS1_PADDING,        OSSL_PKEY_RSA_PAD_MODE_PKCSV15 },
+    { RSA_NO_PADDING,           OSSL_PKEY_RSA_PAD_MODE_NONE },
+    { RSA_PKCS1_OAEP_PADDING,   OSSL_PKEY_RSA_PAD_MODE_OAEP }, /* Correct spelling first */
+    { RSA_PKCS1_OAEP_PADDING,   "oeap"   },
+    { RSA_X931_PADDING,         OSSL_PKEY_RSA_PAD_MODE_X931 },
+    { 0,                        NULL     }
 };
 
 /*
@@ -75,6 +75,8 @@ typedef struct {
     /* TLS padding */
     unsigned int client_version;
     unsigned int alt_version;
+    /* PKCS#1 v1.5 decryption mode */
+    unsigned int implicit_rejection;
 } PROV_RSA_CTX;
 
 static void *rsa_newctx(void *provctx)
@@ -92,7 +94,7 @@ static void *rsa_newctx(void *provctx)
 }
 
 static int rsa_init(void *vprsactx, void *vrsa, const OSSL_PARAM params[],
-    int operation)
+                    int operation)
 {
     PROV_RSA_CTX *prsactx = (PROV_RSA_CTX *)vprsactx;
 
@@ -107,6 +109,7 @@ static int rsa_init(void *vprsactx, void *vrsa, const OSSL_PARAM params[],
     RSA_free(prsactx->rsa);
     prsactx->rsa = vrsa;
     prsactx->operation = operation;
+    prsactx->implicit_rejection = 1;
 
     switch (RSA_test_flags(prsactx->rsa, RSA_FLAG_TYPE_MASK)) {
     case RSA_FLAG_TYPE_RSA:
@@ -121,19 +124,19 @@ static int rsa_init(void *vprsactx, void *vrsa, const OSSL_PARAM params[],
 }
 
 static int rsa_encrypt_init(void *vprsactx, void *vrsa,
-    const OSSL_PARAM params[])
+                            const OSSL_PARAM params[])
 {
     return rsa_init(vprsactx, vrsa, params, EVP_PKEY_OP_ENCRYPT);
 }
 
 static int rsa_decrypt_init(void *vprsactx, void *vrsa,
-    const OSSL_PARAM params[])
+                            const OSSL_PARAM params[])
 {
     return rsa_init(vprsactx, vrsa, params, EVP_PKEY_OP_DECRYPT);
 }
 
 static int rsa_encrypt(void *vprsactx, unsigned char *out, size_t *outlen,
-    size_t outsize, const unsigned char *in, size_t inlen)
+                       size_t outsize, const unsigned char *in, size_t inlen)
 {
     PROV_RSA_CTX *prsactx = (PROV_RSA_CTX *)vprsactx;
     size_t len = RSA_size(prsactx->rsa);
@@ -161,10 +164,8 @@ static int rsa_encrypt(void *vprsactx, unsigned char *out, size_t *outlen,
         int rsasize = RSA_size(prsactx->rsa);
         unsigned char *tbuf;
 
-        if ((tbuf = OPENSSL_malloc(rsasize)) == NULL) {
-            ERR_raise(ERR_LIB_PROV, ERR_R_MALLOC_FAILURE);
+        if ((tbuf = OPENSSL_malloc(rsasize)) == NULL)
             return 0;
-        }
         if (prsactx->oaep_md == NULL) {
             prsactx->oaep_md = EVP_MD_fetch(prsactx->libctx, "SHA-1", NULL);
             if (prsactx->oaep_md == NULL) {
@@ -173,23 +174,24 @@ static int rsa_encrypt(void *vprsactx, unsigned char *out, size_t *outlen,
                 return 0;
             }
         }
-        ret = ossl_rsa_padding_add_PKCS1_OAEP_mgf1_ex(prsactx->libctx, tbuf,
-            rsasize, in, inlen,
-            prsactx->oaep_label,
-            prsactx->oaep_labellen,
-            prsactx->oaep_md,
-            prsactx->mgf1_md);
+        ret =
+            ossl_rsa_padding_add_PKCS1_OAEP_mgf1_ex(prsactx->libctx, tbuf,
+                                                    rsasize, in, inlen,
+                                                    prsactx->oaep_label,
+                                                    prsactx->oaep_labellen,
+                                                    prsactx->oaep_md,
+                                                    prsactx->mgf1_md);
 
         if (!ret) {
             OPENSSL_free(tbuf);
             return 0;
         }
         ret = RSA_public_encrypt(rsasize, tbuf, out, prsactx->rsa,
-            RSA_NO_PADDING);
+                                 RSA_NO_PADDING);
         OPENSSL_free(tbuf);
     } else {
         ret = RSA_public_encrypt(inlen, in, out, prsactx->rsa,
-            prsactx->pad_mode);
+                                 prsactx->pad_mode);
     }
     /* A ret value of 0 is not an error */
     if (ret < 0)
@@ -199,10 +201,11 @@ static int rsa_encrypt(void *vprsactx, unsigned char *out, size_t *outlen,
 }
 
 static int rsa_decrypt(void *vprsactx, unsigned char *out, size_t *outlen,
-    size_t outsize, const unsigned char *in, size_t inlen)
+                       size_t outsize, const unsigned char *in, size_t inlen)
 {
     PROV_RSA_CTX *prsactx = (PROV_RSA_CTX *)vprsactx;
     int ret;
+    int pad_mode;
     size_t len = RSA_size(prsactx->rsa);
 
     if (!ossl_prov_is_running())
@@ -234,15 +237,13 @@ static int rsa_decrypt(void *vprsactx, unsigned char *out, size_t *outlen,
     }
 
     if (prsactx->pad_mode == RSA_PKCS1_OAEP_PADDING
-        || prsactx->pad_mode == RSA_PKCS1_WITH_TLS_PADDING) {
+            || prsactx->pad_mode == RSA_PKCS1_WITH_TLS_PADDING) {
         unsigned char *tbuf;
 
-        if ((tbuf = OPENSSL_malloc(len)) == NULL) {
-            ERR_raise(ERR_LIB_PROV, ERR_R_MALLOC_FAILURE);
+        if ((tbuf = OPENSSL_malloc(len)) == NULL)
             return 0;
-        }
         ret = RSA_private_decrypt(inlen, in, tbuf, prsactx->rsa,
-            RSA_NO_PADDING);
+                                  RSA_NO_PADDING);
         /*
          * With no padding then, on success ret should be len, otherwise an
          * error occurred (non-constant time)
@@ -262,11 +263,11 @@ static int rsa_decrypt(void *vprsactx, unsigned char *out, size_t *outlen,
                 }
             }
             ret = RSA_padding_check_PKCS1_OAEP_mgf1(out, outsize, tbuf,
-                len, len,
-                prsactx->oaep_label,
-                prsactx->oaep_labellen,
-                prsactx->oaep_md,
-                prsactx->mgf1_md);
+                                                    len, len,
+                                                    prsactx->oaep_label,
+                                                    prsactx->oaep_labellen,
+                                                    prsactx->oaep_md,
+                                                    prsactx->mgf1_md);
         } else {
             /* RSA_PKCS1_WITH_TLS_PADDING */
             if (prsactx->client_version <= 0) {
@@ -275,13 +276,17 @@ static int rsa_decrypt(void *vprsactx, unsigned char *out, size_t *outlen,
                 return 0;
             }
             ret = ossl_rsa_padding_check_PKCS1_type_2_TLS(
-                prsactx->libctx, out, outsize, tbuf, len,
-                prsactx->client_version, prsactx->alt_version);
+                        prsactx->libctx, out, outsize, tbuf, len,
+                        prsactx->client_version, prsactx->alt_version);
         }
         OPENSSL_free(tbuf);
     } else {
-        ret = RSA_private_decrypt(inlen, in, out, prsactx->rsa,
-            prsactx->pad_mode);
+        if ((prsactx->implicit_rejection == 0) &&
+                (prsactx->pad_mode == RSA_PKCS1_PADDING))
+            pad_mode = RSA_PKCS1_NO_IMPLICIT_REJECT_PADDING;
+        else
+            pad_mode = prsactx->pad_mode;
+        ret = RSA_private_decrypt(inlen, in, out, prsactx->rsa, pad_mode);
     }
     *outlen = constant_time_select_s(constant_time_msb_s(ret), *outlen, ret);
     ret = constant_time_select_int(constant_time_msb(ret), 0, 1);
@@ -332,12 +337,6 @@ static void *rsa_dupctx(void *vprsactx)
         return NULL;
     }
 
-    if (dstctx->oaep_label != NULL
-        && (dstctx->oaep_label = OPENSSL_memdup(dstctx->oaep_label, dstctx->oaep_labellen)) == NULL) {
-        rsa_freectx(dstctx);
-        return NULL;
-    }
-
     return dstctx;
 }
 
@@ -356,30 +355,34 @@ static int rsa_get_ctx_params(void *vprsactx, OSSL_PARAM *params)
             if (!OSSL_PARAM_set_int(p, prsactx->pad_mode))
                 return 0;
             break;
-        case OSSL_PARAM_UTF8_STRING: {
-            int i;
-            const char *word = NULL;
+        case OSSL_PARAM_UTF8_STRING:
+            {
+                int i;
+                const char *word = NULL;
 
-            for (i = 0; padding_item[i].id != 0; i++) {
-                if (prsactx->pad_mode == (int)padding_item[i].id) {
-                    word = padding_item[i].ptr;
-                    break;
+                for (i = 0; padding_item[i].id != 0; i++) {
+                    if (prsactx->pad_mode == (int)padding_item[i].id) {
+                        word = padding_item[i].ptr;
+                        break;
+                    }
+                }
+
+                if (word != NULL) {
+                    if (!OSSL_PARAM_set_utf8_string(p, word))
+                        return 0;
+                } else {
+                    ERR_raise(ERR_LIB_PROV, ERR_R_INTERNAL_ERROR);
                 }
             }
-
-            if (word != NULL) {
-                if (!OSSL_PARAM_set_utf8_string(p, word))
-                    return 0;
-            } else {
-                ERR_raise(ERR_LIB_PROV, ERR_R_INTERNAL_ERROR);
-            }
-        } break;
+            break;
         default:
             return 0;
         }
 
     p = OSSL_PARAM_locate(params, OSSL_ASYM_CIPHER_PARAM_OAEP_DIGEST);
-    if (p != NULL && !OSSL_PARAM_set_utf8_string(p, prsactx->oaep_md == NULL ? "" : EVP_MD_get0_name(prsactx->oaep_md)))
+    if (p != NULL && !OSSL_PARAM_set_utf8_string(p, prsactx->oaep_md == NULL
+                                                    ? ""
+                                                    : EVP_MD_get0_name(prsactx->oaep_md)))
         return 0;
 
     p = OSSL_PARAM_locate(params, OSSL_ASYM_CIPHER_PARAM_MGF1_DIGEST);
@@ -387,12 +390,16 @@ static int rsa_get_ctx_params(void *vprsactx, OSSL_PARAM *params)
         EVP_MD *mgf1_md = prsactx->mgf1_md == NULL ? prsactx->oaep_md
                                                    : prsactx->mgf1_md;
 
-        if (!OSSL_PARAM_set_utf8_string(p, mgf1_md == NULL ? "" : EVP_MD_get0_name(mgf1_md)))
-            return 0;
+        if (!OSSL_PARAM_set_utf8_string(p, mgf1_md == NULL
+                                           ? ""
+                                           : EVP_MD_get0_name(mgf1_md)))
+        return 0;
     }
 
     p = OSSL_PARAM_locate(params, OSSL_ASYM_CIPHER_PARAM_OAEP_LABEL);
-    if (p != NULL && !OSSL_PARAM_set_octet_ptr(p, prsactx->oaep_label, prsactx->oaep_labellen))
+    if (p != NULL &&
+        !OSSL_PARAM_set_octet_ptr(p, prsactx->oaep_label,
+                                  prsactx->oaep_labellen))
         return 0;
 
     p = OSSL_PARAM_locate(params, OSSL_ASYM_CIPHER_PARAM_TLS_CLIENT_VERSION);
@@ -403,6 +410,10 @@ static int rsa_get_ctx_params(void *vprsactx, OSSL_PARAM *params)
     if (p != NULL && !OSSL_PARAM_set_uint(p, prsactx->alt_version))
         return 0;
 
+    p = OSSL_PARAM_locate(params, OSSL_ASYM_CIPHER_PARAM_IMPLICIT_REJECTION);
+    if (p != NULL && !OSSL_PARAM_set_uint(p, prsactx->implicit_rejection))
+        return 0;
+
     return 1;
 }
 
@@ -411,14 +422,15 @@ static const OSSL_PARAM known_gettable_ctx_params[] = {
     OSSL_PARAM_utf8_string(OSSL_ASYM_CIPHER_PARAM_PAD_MODE, NULL, 0),
     OSSL_PARAM_utf8_string(OSSL_ASYM_CIPHER_PARAM_MGF1_DIGEST, NULL, 0),
     OSSL_PARAM_DEFN(OSSL_ASYM_CIPHER_PARAM_OAEP_LABEL, OSSL_PARAM_OCTET_PTR,
-        NULL, 0),
+                    NULL, 0),
     OSSL_PARAM_uint(OSSL_ASYM_CIPHER_PARAM_TLS_CLIENT_VERSION, NULL),
     OSSL_PARAM_uint(OSSL_ASYM_CIPHER_PARAM_TLS_NEGOTIATED_VERSION, NULL),
+    OSSL_PARAM_uint(OSSL_ASYM_CIPHER_PARAM_IMPLICIT_REJECTION, NULL),
     OSSL_PARAM_END
 };
 
 static const OSSL_PARAM *rsa_gettable_ctx_params(ossl_unused void *vprsactx,
-    ossl_unused void *provctx)
+                                                 ossl_unused void *provctx)
 {
     return known_gettable_ctx_params;
 }
@@ -443,7 +455,7 @@ static int rsa_set_ctx_params(void *vprsactx, const OSSL_PARAM params[])
             return 0;
 
         p = OSSL_PARAM_locate_const(params,
-            OSSL_ASYM_CIPHER_PARAM_OAEP_DIGEST_PROPS);
+                                    OSSL_ASYM_CIPHER_PARAM_OAEP_DIGEST_PROPS);
         if (p != NULL) {
             str = mdprops;
             if (!OSSL_PARAM_get_utf8_string(p, &str, sizeof(mdprops)))
@@ -466,19 +478,21 @@ static int rsa_set_ctx_params(void *vprsactx, const OSSL_PARAM params[])
             if (!OSSL_PARAM_get_int(p, &pad_mode))
                 return 0;
             break;
-        case OSSL_PARAM_UTF8_STRING: {
-            int i;
+        case OSSL_PARAM_UTF8_STRING:
+            {
+                int i;
 
-            if (p->data == NULL)
-                return 0;
+                if (p->data == NULL)
+                    return 0;
 
-            for (i = 0; padding_item[i].id != 0; i++) {
-                if (strcmp(p->data, padding_item[i].ptr) == 0) {
-                    pad_mode = padding_item[i].id;
-                    break;
+                for (i = 0; padding_item[i].id != 0; i++) {
+                    if (strcmp(p->data, padding_item[i].ptr) == 0) {
+                        pad_mode = padding_item[i].id;
+                        break;
+                    }
                 }
             }
-        } break;
+            break;
         default:
             return 0;
         }
@@ -504,7 +518,7 @@ static int rsa_set_ctx_params(void *vprsactx, const OSSL_PARAM params[])
             return 0;
 
         p = OSSL_PARAM_locate_const(params,
-            OSSL_ASYM_CIPHER_PARAM_MGF1_DIGEST_PROPS);
+                                    OSSL_ASYM_CIPHER_PARAM_MGF1_DIGEST_PROPS);
         if (p != NULL) {
             str = mdprops;
             if (!OSSL_PARAM_get_utf8_string(p, &str, sizeof(mdprops)))
@@ -549,6 +563,14 @@ static int rsa_set_ctx_params(void *vprsactx, const OSSL_PARAM params[])
             return 0;
         prsactx->alt_version = alt_version;
     }
+    p = OSSL_PARAM_locate_const(params, OSSL_ASYM_CIPHER_PARAM_IMPLICIT_REJECTION);
+    if (p != NULL) {
+        unsigned int implicit_rejection;
+
+        if (!OSSL_PARAM_get_uint(p, &implicit_rejection))
+            return 0;
+        prsactx->implicit_rejection = implicit_rejection;
+    }
 
     return 1;
 }
@@ -562,11 +584,12 @@ static const OSSL_PARAM known_settable_ctx_params[] = {
     OSSL_PARAM_octet_string(OSSL_ASYM_CIPHER_PARAM_OAEP_LABEL, NULL, 0),
     OSSL_PARAM_uint(OSSL_ASYM_CIPHER_PARAM_TLS_CLIENT_VERSION, NULL),
     OSSL_PARAM_uint(OSSL_ASYM_CIPHER_PARAM_TLS_NEGOTIATED_VERSION, NULL),
+    OSSL_PARAM_uint(OSSL_ASYM_CIPHER_PARAM_IMPLICIT_REJECTION, NULL),
     OSSL_PARAM_END
 };
 
 static const OSSL_PARAM *rsa_settable_ctx_params(ossl_unused void *vprsactx,
-    ossl_unused void *provctx)
+                                                 ossl_unused void *provctx)
 {
     return known_settable_ctx_params;
 }
@@ -580,12 +603,12 @@ const OSSL_DISPATCH ossl_rsa_asym_cipher_functions[] = {
     { OSSL_FUNC_ASYM_CIPHER_FREECTX, (void (*)(void))rsa_freectx },
     { OSSL_FUNC_ASYM_CIPHER_DUPCTX, (void (*)(void))rsa_dupctx },
     { OSSL_FUNC_ASYM_CIPHER_GET_CTX_PARAMS,
-        (void (*)(void))rsa_get_ctx_params },
+      (void (*)(void))rsa_get_ctx_params },
     { OSSL_FUNC_ASYM_CIPHER_GETTABLE_CTX_PARAMS,
-        (void (*)(void))rsa_gettable_ctx_params },
+      (void (*)(void))rsa_gettable_ctx_params },
     { OSSL_FUNC_ASYM_CIPHER_SET_CTX_PARAMS,
-        (void (*)(void))rsa_set_ctx_params },
+      (void (*)(void))rsa_set_ctx_params },
     { OSSL_FUNC_ASYM_CIPHER_SETTABLE_CTX_PARAMS,
-        (void (*)(void))rsa_settable_ctx_params },
-    { 0, NULL }
+      (void (*)(void))rsa_settable_ctx_params },
+    OSSL_DISPATCH_END
 };
