@@ -1,5 +1,5 @@
 /*
- * Copyright 2022-2023 The OpenSSL Project Authors. All Rights Reserved.
+ * Copyright 2022-2024 The OpenSSL Project Authors. All Rights Reserved.
  *
  * Licensed under the Apache License 2.0 (the "License").  You may not use
  * this file except in compliance with the License.  You can obtain a copy
@@ -363,8 +363,8 @@ static const struct rx_test_op rx_script_1[] = {
     RX_OP_STEP_TIME(250 * OSSL_TIME_MS)
     RX_OP_RX(0, INIT_WINDOW_SIZE * 5 + 1, 0)
     RX_OP_CHECK_SWM(INIT_WINDOW_SIZE * 5)
-    RX_OP_CHECK_ERROR(QUIC_ERR_FLOW_CONTROL_ERROR, 0)
-    RX_OP_CHECK_ERROR(QUIC_ERR_FLOW_CONTROL_ERROR, 1)
+    RX_OP_CHECK_ERROR(OSSL_QUIC_ERR_FLOW_CONTROL_ERROR, 0)
+    RX_OP_CHECK_ERROR(OSSL_QUIC_ERR_FLOW_CONTROL_ERROR, 1)
     RX_OP_CHECK_ERROR(0, 0)
     RX_OP_CHECK_CWM(INIT_WINDOW_SIZE * 5)
     /* 
@@ -456,15 +456,15 @@ static const struct rx_test_op rx_script_2[] = {
 
     /* Test exceeding limit at stream level. */
     RX_OP_RX(0, INIT_S_WINDOW_SIZE * 2 + 1, 0)
-    RX_OP_CHECK_ERROR_STREAM(0, QUIC_ERR_FLOW_CONTROL_ERROR, 0)
-    RX_OP_CHECK_ERROR_STREAM(0, QUIC_ERR_FLOW_CONTROL_ERROR, 1)
+    RX_OP_CHECK_ERROR_STREAM(0, OSSL_QUIC_ERR_FLOW_CONTROL_ERROR, 0)
+    RX_OP_CHECK_ERROR_STREAM(0, OSSL_QUIC_ERR_FLOW_CONTROL_ERROR, 1)
     RX_OP_CHECK_ERROR_STREAM(0, 0, 0)
     RX_OP_CHECK_ERROR_CONN(0, 0) /* doesn't affect conn */
 
     /* Test exceeding limit at connection level. */
     RX_OP_RX(0, INIT_WINDOW_SIZE * 2, 0)
-    RX_OP_CHECK_ERROR_CONN(QUIC_ERR_FLOW_CONTROL_ERROR, 0)
-    RX_OP_CHECK_ERROR_CONN(QUIC_ERR_FLOW_CONTROL_ERROR, 1)
+    RX_OP_CHECK_ERROR_CONN(OSSL_QUIC_ERR_FLOW_CONTROL_ERROR, 0)
+    RX_OP_CHECK_ERROR_CONN(OSSL_QUIC_ERR_FLOW_CONTROL_ERROR, 1)
     RX_OP_CHECK_ERROR_CONN(0, 0)
 
     RX_OP_END
